'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var fusionCore = require('fusion-core');
var fusionTokens = require('fusion-tokens');

/** Copyright (c) 2018 Uber Technologies, Inc.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 *
 * 
 */
var I18nLoaderToken = fusionCore.createToken('I18nLoaderToken');

/** Copyright (c) 2018 Uber Technologies, Inc.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 *
 * 
 */
var createLoader = null;

var _templateObject = /*#__PURE__*/ _taggedTemplateLiteralLoose(["\n            <script type=\"application/json\" id=\"__TRANSLATIONS__\">\n              ", "\n            </script>\n          "], ["\n            <script type=\"application/json\" id=\"__TRANSLATIONS__\">\n              ", "\n            </script>\n          "]);

function _taggedTemplateLiteralLoose(strings, raw) { if (!raw) { raw = strings.slice(0); } strings.raw = raw; return strings; }

/** Copyright (c) 2018 Uber Technologies, Inc.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 *
 * 
 */

/* eslint-env node */
// exported for testing

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; var ownKeys = Object.keys(source); if (typeof Object.getOwnPropertySymbols === 'function') { ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) { return Object.getOwnPropertyDescriptor(source, sym).enumerable; })); } ownKeys.forEach(function (key) { _defineProperty(target, key, source[key]); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

/** Copyright (c) 2018 Uber Technologies, Inc.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 *
 * 
 */

/* eslint-env browser */
function loadTranslations() {
  var element = document.getElementById('__TRANSLATIONS__');

  if (!element) {
    throw new Error('[fusion-plugin-i18n] - Could not find a __TRANSLATIONS__ element');
  }

  try {
    return JSON.parse(fusionCore.unescape(element.textContent));
  } catch (e) {
    throw new Error('[fusion-plugin-i18n] - Error parsing __TRANSLATIONS__ element content');
  }
}

var HydrationStateToken = fusionCore.createToken('HydrationStateToken');

var pluginFactory$1 = function pluginFactory() {
  return fusionCore.createPlugin({
    deps: {
      fetch: fusionTokens.FetchToken.optional,
      hydrationState: HydrationStateToken.optional
    },
    provides: function provides(_temp) {
      var _ref = _temp === void 0 ? {} : _temp,
          _ref$fetch = _ref.fetch,
          fetch = _ref$fetch === void 0 ? window.fetch : _ref$fetch,
          hydrationState = _ref.hydrationState;

      var I18n =
      /*#__PURE__*/
      function () {
        function I18n() {
          var _ref2 = hydrationState || loadTranslations(),
              localeCode = _ref2.localeCode,
              translations = _ref2.translations;

          this.requestedKeys = new Set();
          this.localeCode = localeCode;
          this.translations = translations || {};
        }

        var _proto = I18n.prototype;

        _proto.load = function load(translationKeys) {
          return new Promise(function ($return, $error) {
            var _this = this;

            var loadedKeys = Object.keys(this.translations);
            var unloaded = translationKeys.filter(function (key) {
              return loadedKeys.indexOf(key) < 0 && !_this.requestedKeys.has(key);
            });
            var fetchOpts = {
              method: 'POST',
              headers: _objectSpread({
                Accept: '*/*'
              }, this.localeCode ? {
                'X-Fusion-Locale-Code': this.localeCode
              } : {})
            };

            if (unloaded.length > 0) {
              // Don't try to load translations again if a request is already in
              // flight. This means that we need to add unloaded chunks to
              // loadedChunks optimistically and remove them if some error happens
              unloaded.forEach(function (key) {
                _this.requestedKeys.add(key);
              }); // TODO(#3) don't append prefix if injected fetch also injects prefix

              return $return(fetch("/_translations?keys=" + JSON.stringify(unloaded), fetchOpts).then(function (r) {
                return r.json();
              }).then(function (data) {
                for (var key in data) {
                  _this.translations[key] = data[key];

                  _this.requestedKeys.delete(key);
                }
              }).catch(function (err) {
                // An error occurred, so remove the chunks we were trying to load
                // from loadedChunks. This allows us to try to load those chunk
                // translations again
                unloaded.forEach(function (key) {
                  _this.requestedKeys.delete(key);
                });
                throw err;
              }));
            }

            return $return();
          }.bind(this));
        };

        _proto.translate = function translate(key, interpolations) {
          if (interpolations === void 0) {
            interpolations = {};
          }

          var template = this.translations[key];
          return template ? template.replace(/\${(.*?)}/g, function (_, k) {
            return interpolations[k] === void 0 ? '${' + k + '}' : interpolations[k];
          }) : key;
        };

        return I18n;
      }();

      var i18n = new I18n();
      return {
        from: function from() {
          return i18n;
        }
      };
    }
  });
};

var clientPlugin = true && pluginFactory$1();

/** Copyright (c) 2018 Uber Technologies, Inc.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 *
 * 
 */
var I18nToken = fusionCore.createToken('I18nToken');
var plugin = clientPlugin;

exports.default = plugin;
exports.I18nToken = I18nToken;
exports.I18nLoaderToken = I18nLoaderToken;
exports.HydrationStateToken = HydrationStateToken;
exports.createI18nLoader = createLoader;
